using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.UI;

namespace Common.Web.Modules
{
    /// <summary>
    /// Copyright: Adam Langley
    /// This HTTP module will place a banner at on all ASPX pages if they are compiled
    /// in Debug mode.
    /// The banner image, and ALT resource text is stored in this assembly.
    /// </summary>
    public class BuildConfigurationBannerModule : IHttpModule
    {
        private HttpApplication application = null;

        #region IHttpModule Members

        public void Dispose()
        {
            // deregister the event handler
            application.PreRequestHandlerExecute -= new EventHandler(application_PreRequestHandlerExecute);
        }

        public void Init(HttpApplication context)
        {
            this.application = context;

            // register to receive events from the HTTP pipeline
            context.PreRequestHandlerExecute += new EventHandler(application_PreRequestHandlerExecute);
        }

        void application_PreRequestHandlerExecute(object sender, EventArgs e)
        {
            HttpApplication application = (HttpApplication)sender;
            AttachBanner(application);    
        }

        protected virtual void AttachBanner(HttpApplication application)
        {
            IHttpHandler handler = application.Context.Handler;
            if (handler is Page)
            {
                Page page = handler as Page;
                if (IsAssemblyDebugBuild(page))
                {
                    page.ClientScript.RegisterClientScriptInclude("prototype.js", page.ClientScript.GetWebResourceUrl(GetType(), "Common.Web.Resources.prototype.js"));

                    // obtain the version number of the executing assembly, so that it may be used in the banners alternate text 
                    Version version = page.GetType().BaseType.Assembly.GetName().Version;

                    // retrieve the banner HTML from the localized resources, inserting a dynamic URL for the image, and the alternate text
                    string szHtml = string.Format(BuildConfigurationBannerResources.BannerHtml,
                        page.ClientScript.GetWebResourceUrl(GetType(), "Common.Web.Resources.BannerImage.png"), version);

                    // register a javascript that will insert the HTML into the DOM after the page has loaded.
                    page.ClientScript.RegisterStartupScript(GetType(), "DevelopmentBanner",
                        string.Format("Event.observe(window, 'load', function(){{new Insertion.Bottom($(document.body), '{0}');}});", szHtml), true);
                }
            }
        }

        #endregion

        /// <summary>
        /// Determines whether a Page instance is a debug build or not.
        /// </summary>
        /// <param name="page">The Page instance to test</param>
        /// <returns>true or false</returns>
        private bool IsAssemblyDebugBuild(Page page)
        {
            foreach (Attribute att in page.GetType().BaseType.Assembly.GetCustomAttributes(false))
                if (att is System.Diagnostics.DebuggableAttribute)
                    return (att as System.Diagnostics.DebuggableAttribute).IsJITTrackingEnabled;
            return false;
        }

    }
}
